## Placeholder
OS_PIPELINE_ARCHIVED = "Pipeline '{pipeline_id}' archived"
OS_UNABLE_TO_ARCHIVE_PIPELINE = "Unable to archive pipeline '{pipeline_id}'"

OS_PIPELINE_ACTIVATED = "Pipeline '{pipeline_id}' activated"
OS_UNABLE_TO_ACTIVATE_PIPELINE = "Unable to activate pipeline '{pipeline_id}'"

OS_PIPELINE_DISABLED = "Pipeline '{pipeline_id}' disabled"
OS_UNABLE_TO_DISABLE_PIPELINE = "Unable to disable pipeline '{pipeline_id}'"

OS_PIPELINE_ENABLED = "Pipeline '{pipeline_id}' enabled"
OS_UNABLE_TO_ENABLE_PIPELINE = "Unable to enable pipeline '{pipeline_id}'"

OS_PIPELINE_DELETED = "Pipeline '{pipeline_id}' deleted"
OS_UNABLE_TO_DELETE_PIPELINE = "Unable to delete pipeline '{pipeline_id}'"

OS_DEPLOYMENT_TRIGGERED = "Deployment '{deployment_id}' triggered for pipeline '{pipeline_id}' and has a status of '{deployment_status}'"

OS_DEPLOYMENT_ABORTED = "Deployment '{deployment_id}' aborted"

OS_DEPLOYMENT_PAUSED = "Deployment '{deployment_id}' paused"
OS_UNABLE_TO_PAUSE_DEPLOYMENT = "Deployment '{deployment_id}' could not be paused"

OS_DEPLOYMENT_RESUMED = "Deployment '{deployment_id}' resumed"
OS_UNABLE_TO_RESUME_DEPLOYMENT = "Deployment '{deployment_id}' could not be resumed"

OS_DEPLOYMENT_RETRIED = "Deployment '{deployment_id}' retried"
OS_UNABLE_TO_RETRY_DEPLOYMENT = "Deployment '{deployment_id}' could not be retried"

OS_DEPLOYMENT_PROMOTED = "Deployment '{deployment_id}' promoted"
OS_UNABLE_TO_PROMOTE_DEPLOYMENT = "Deployment '{deployment_id}' could not be promoted"

OS_DEPLOYMENT_TASK_SKIPPED = "Deployment task for deployment '{deployment_id}' skipped"
OS_UNABLE_TO_SKIP_DEPLOYMENT_TASK = "Could not skip deployment task for deployment '{deployment_id}'"

OS_WAITING_FOR_DEPLOYMENT_STATUS = "Waiting for deployment '{deployment_id}' status to be one of: {status_watch_list}"
OS_WAITING_FOR_DEPLOYMENT_STATUS_FOUND = "Deployment '{deployment_id}' status is '{deployment_status}' succesfully found match list: '{status_watch_list}'"
OS_WAITING_FOR_DEPLOYMENT_STATUS_NOT_FOUND = "Reached maximum wait time and deployment '{deployment_id}' status '{deployment_status}' is still not in match list '{status_watch_list}'"
OS_WAITING_FOR_DEPLOYMENT_STATUS_PAUSING = "Waiting {poll_frequency_in_seconds} seconds as deployment '{deployment_id}' status '{deployment_status}' is not in must-match list '{status_watch_list}'"

OS_NOT_WAITING_FOR_DEPLOYMENT_STATUS = "Waiting until deployment '{deployment_id}' status changes to something other than: '{status_watch_list}'"
OS_NOT_WAITING_FOR_DEPLOYMENT_STATUS_FOUND = "Deployment '{deployment_id}' status is '{deployment_status}' has succesfully change from: '{status_watch_list}'"
OS_NOT_WAITING_FOR_DEPLOYMENT_STATUS_NOT_FOUND = "Reached maximum wait time and deployment '{deployment_id}' status '{deployment_status}' is still one of: '{status_watch_list}'"
OS_NOT_WAITING_FOR_DEPLOYMENT_STATUS_PAUSING = "Waiting {poll_frequency_in_seconds} seconds as deployment '{deployment_id}' status '{deployment_status}' is present in must-not-match list '{status_watch_list}'"

OS_DEPLOYMENT_TASK_UPDATED = "Task '{task_id}' updated"
OS_UNABLE_TO_UPDATE_DEPLOYMENT_TASK = "Task '{task_id}' could not be updated"

OS_WAITING_FOR_DEPLOYMENT_TASK_STATUS = "Waiting for task '{task_id}' status to be present in must-match list '{status_watch_list}'"
OS_WAITING_FOR_DEPLOYMENT_TASK_STATUS_FOUND = "Task '{task_id}' status is '{task_status}' which is in present in must-match list '{status_watch_list}'"
OS_WAITING_FOR_DEPLOYMENT_TASK_STATUS_NOT_FOUND = "Reached maximum wait time and task '{task_id}' status '{task_status}' is not in must-match list '{status_watch_list}'"
OS_WAITING_FOR_DEPLOYMENT_TASK_STATUS_PAUSING = "Waiting {poll_frequency_in_seconds} seconds as task '{task_id}' status '{task_status}' is not in must-match list {status_watch_list}"

OS_NOT_WAITING_FOR_DEPLOYMENT_TASK_STATUS = "Waiting for task '{task_id}' status to not equal: '{status_watch_list}'"
OS_NOT_WAITING_FOR_DEPLOYMENT_TASK_STATUS_FOUND = "Task '{task_id}' status is '{task_status}' does not equal '{status_watch_list}'"
OS_NOT_WAITING_FOR_DEPLOYMENT_TASK_STATUS_NOT_FOUND = "Reached maximum wait time and task '{task_id}' status '{task_status}' is still present in list: '{status_watch_list}'"
OS_NOT_WAITING_FOR_DEPLOYMENT_TASK_STATUS_PAUSING = "Waiting {poll_frequency_in_seconds} seconds as task '{task_id}' status '{task_status}' is still present in must-not-match list '{status_watch_list}'"

OS_AUTO_DEPLOY_TRIGGER_DEPLOYMENT_FAILED = "Unable to trigger deployment for pipeline '{pipeline_id}', deployment failed"
OS_AUTO_DEPLOY_GET_LATEST_DEPLOYMENT_TASK_FAILED = "Unable to retrieve latest task information, deployment failed"
ES_AUTO_DEPLOY_GET_DEPLOYMENT_TASK_FAILED = "Unable to retrieve task information, deployment failed"
OS_AUTO_DEPLOY_WAITING_STATUS_NOT_OPEN_QUEUED = "Deployment '{task_id}; is queued, waiting for a maximum {max_execution_time_in_seconds} seconds  until it's no longer queued"
OS_AUTO_DEPLOY_WAITING_STATUS_NOT_OPEN_READY = "Current task is ready for background processing, will await pipeline status change"
OS_AUTO_DEPLOY_WAITING_STATUS_NOT_OPEN_INPROGRESS = "Current task still in progress, will await pipeline status change"
OS_AUTO_DEPLOY_TASK_STATUS_RESOLVED_COMPLETED = "Current task is completed, will await pipeline status change"
OS_AUTO_DEPLOY_TASK_AUTO_SKIPPED = "Auto-Skipping task '{task_id}' of type '{task_type}' as requested"
OS_AUTO_DEPLOY_UNABLE_TO_AUTO_SKIP_TASK = "Unable to auto-skip task '{task_id}' of type '{task_type} as requested, deployment failed"
OS_AUTO_DEPLOY_TASK_MAX_RETRIES_REACHED = "Task '{task_id}' has been retried the maximum number of {retries} retries, deployment failed"
OS_AUTO_DEPLOY_TASK_AUTO_RETRIED = "Task in pipeline has failed, attempting automatic rety {retry} of {retries}"
OS_AUTO_DEPLOY_UNABLE_TO_AUTO_RETRY_TASK = "Unable to retry task, deployment failed"
OS_AUTO_DEPLOY_UNKNOWN_ERROR_TASK_TYPE = "Deployment task '{task_type}' has not been configured as an auto-skip or auto-retry task type, awaiting manual progression until timeout"
OS_AUTO_DEPLOY_TASK_AUTO_SKIP_AGED_UPDATES = "Auto-Skipping aged updates task '{task_id}' of type '{task_type}' as requested"
OS_AUTO_DEPLOY_UNABLE_TO_AUTO_SKIP_AGED_UPDATES_TASK = "Unable to auto-skip task '{task_id}' of type '{task_type} as requested, deployment failed"
OS_AUTO_DEPLOY_TASK_AUTO_OVERRIDE_UPDATES = "Auto-Overriding aged updates task '{task_id}' of type '{task_type}' as requested"
OS_AUTO_DEPLOY_UNABLE_TO_AUTO_OVERRIDE_AGED_UPDATES_TASK = "Unable to auto-overrides aged updates for '{task_id}' of type '{task_type} as requested, deployment failed"
OS_AUTO_DEPLOY_TASK_AUTO_REJECT = "Auto-Rejecting task '{task_id}' of type '{task_type}' as requested"
OS_AUTO_DEPLOY_UNABLE_TO_AUTO_REJECT_TASK = "Unable to auto-reject task '{task_id}' of type '{task_type} as requested, deployment failed"
OS_AUTO_DEPLOY_TASK_AUTO_APPROVE = "Auto-Approving task '{task_id}' of type '{task_type}' as requested"
OS_AUTO_DEPLOY_UNABLE_TO_AUTO_APPROVE_TASK = "Unable to auto-approve task '{task_id}' of type '{task_type} as requested, deployment failed"
OS_AUTO_DEPLOY_UNKNOWN_INPUT_TASK_TYPE = "Deployment task '{task_type}' has not been configured as an auto-approve or auto-reject task type, awaiting manual progression until timeout"
OS_AUTO_DEPLOY_UNKNOWN_TASK_STATUS = "Unrecognised task status '{task_status}' so unable to proceed, deployment failed"
OS_AUTO_DEPLOY_SUCCESS = "Deployment has been succesfully completed"
OS_AUTO_DEPLOY_RESOLVED_WITH_ERROR = "Deployment has failed with an error"
OS_AUTO_DEPLOY_UNKNOWN_DEPLOYMENT_STATUS = "Unrecognised deployment status {deployment_status}"
OS_AUTO_DEPLOY_MAX_EXECUTION_TIME_EXCEEDED = "Max execution time exceeded and pipeline not resolved, deployment failed"


import pyPega.py

## Orchestrator initalisation
orchestrator = pyPega.DeploymentManagerOrchestrator.Orchestrator("orchestrator", "https://sleij.pegatsdemo.com/prweb")

orchestrator.set_api_response_logging_level(0)

## OAuth 2.0 authentication for Orchestrator environment
orchestrator.set_authentication_method(
    orchestrator.AUTHENTICATION_METHOD_OAUTH2_CLIENT_CREDENTIALS,
    "10825762515797159851",
    "A65775712FFB4402D9104A822E9D7CD9",
)
##orchestrator.set_api_response_logging_level(1)

## Abort all existing deployments and tigger a new one
orchestrator.abort_all_deployments_for_pipeline(
    "Pipeline-CLX19", "Starting fresh deployments"
)

## Provide values for execution
pipeline_id = "Pipeline-CLX19"
deployment_reason = "Python testing"
triggered_by = "rob.smart@pega.com"
triggered_by_name = "Rob Smart"
branch_name = ""
max_execution_time_in_seconds = 1800
skip_aged_updates = True
max_single_task_retries = 3
auto_retry_task_types = [
    orchestrator.DEPLOYMENT_TASK_TYPE_GENERATE_ARTIFACT
]
auto_skip_task_types = [
    orchestrator.DEPLOYMENT_TASK_TYPE_VERIFY_SECURITY_CHECKLIST,
    orchestrator.DEPLOYMENT_TASK_TYPE_CHECK_GUARDRAIL_COMPLIANCE,
]
auto_approve_task_types = [
    orchestrator.DEPLOYMENT_TASK_TYPE_PERFORM_MANUAL_STEP
]
auto_reject_task_types = [
    
]
abort_on_failure = True

## Run an automated deployment
if not orchestrator.automate_deployment(
    pipeline_id,
    deployment_reason,
    triggered_by,
    triggered_by_name,
    branch_name,
    max_execution_time_in_seconds,
    max_single_task_retries,
    skip_aged_updates,
    auto_retry_task_types,
    auto_skip_task_types,
    auto_approve_task_types,
    auto_reject_task_types,
    abort_on_failure,
):
    sys.exit(1)
else:
    sys.exit(0)



## Candidate - Application endpoints
API_ENDPOINT_CANDIDATE_GET_RULESET = "/api/cicdext/v1/ruleset/{ruleset_name}/{ruleset_version}"
API_ENDPOINT_CANDIDATE_GET_APPLICATION = "/api/cicdext/v1/application/{appName}/{appVersion}"
API_ENDPOINT_CANDIDATE_CREATE_RULESET = "/api/cicdext/v1/ruleset/{ruleset_name}/{ruleset_version}"
API_ENDPOINT_CANDIDATE_GET_ACCESSGORUPS = "/api/cicdext/v1/accessgroups"


## Environment - __print_api_response()
OS_PRINT_API_RESPONSE = "API call to '{method} {url}' returned status HTTP {status_code}"

## Environment - set_logging_level()
OS_API_RESPONSE_LOGGING_LEVEL_SET = "API response logging level set to '{level}'"

## Environment - set_authentication_method()
OS_AUTHENTICATION_METHOD_SET = "Authentication method set to '{authentication_method}'"
OS_AUTHENTICATION_METHOD_INVALID = "Invalid authentication method '{authentication_method} provided"

## Environment - set_base_url()
OS_ENVIRONMENT_URL_SET = "Base URL set to '{url}'"
OS_URL_MUST_START_WITH = "URL must begin with '{must_start_with}' but was specified as '{url}'"
OS_URL_MUST_END_WITH = "URL must end with '{must_end_with}' but was specified as '{url}'"

## Environment - set_application_context()
OS_APPLICATION_CONTEXT_SET = "Application context set to '{application_context}', please ensure the corresponding alias is configured in Pega"

## Environment - set_oauth_authorization_token_url()
OS_OAUTH_AUTHORIZATION_TOKEN_URL_MUST_START_WITH = "Authorizaton token URL must begin with '{must_start_with}' but was specified as '{authorization_token_url}'"
OS_OAUTH_AUTHORIZATION_TOKEN_URL_MUST_END_WITH = "Authorizaton token URL must end with '{must_end_with}' but was specified as '{authorization_token_url}'"
OS_OAUTH_AUTHORIZATION_TOKEN_URL_SET = "OAuth authorization token URL set to {authorization_token_url}'"

## Environment - set_oauth_authorization_client_id()
OS_OAUTH_CLIENT_ID_SET = "OAuth client ID set to '{client_id}'"

## Environment - set_oauth_client_secret()
OS_OAUTH_CLIENT_SECRET_SET = "OAuth client secret set"

## Environment - set_operator()
OS_OPERATOR_SET = "Operator set as '{operator}'"

## Environment - set_operator_password()
OS_OPERATOR_PASSWORD_SET = "Operator password set"

## Environment - obtain_oauth_token()
OS_OAUTH_TOKEN_OBTAINED = "OAuth token obtained. Token expires at '{token_expires_at}'"

## Environment - Other Oatuh problems
OS_UNABLE_TO_RETRIEVE_OAUTH_TOKEN = "Unable to obtain OAuth token"
OS_AUTHENTICATION_METHOD_MISSING_OR_INVALID = "Authentication method missing or invalid. Use set_authentication_method() method."
OS_OAUTH_AUTHORIZATION_TOKEN_URL_MISSING = "Cannot obtain token as token URL not provided. Use set_oauth_authorization_token_url() method."
OS_OAUTH_CLIENT_ID_MISSING = "Cannot obtain token as client ID not provided. Use set_oauth_client_id() method."
OS_OAUTH_CLIENT_SECRET_MISSING = "Cannot obtain token as client ID not provided. Use set_oauth_client_id() method."




## Orchestrator - Pipeline endpoints
API_ENDPOINT_ORCHESTRATOR_GET_PIPELINE = "/api/DeploymentManager/v1/pipelines/{pipelineid}"
API_ENDPOINT_ORCHESTRATOR_GET_PIPELINES = "/api/DeploymentManager/v1/pipelines/{pipelineid}"
API_ENDPOINT_ORCHESTRATOR_ACTIVATE_PIPELINE = "/api/DeploymentManager/v1/pipelines/{pipelineid}/activate"
API_ENDPOINT_ORCHESTRATOR_ARCHIVE_PIPELINE = "/api/DeploymentManager/v1/pipelines/{pipelineid}/archive"
API_ENDPOINT_ORCHESTRATOR_DISABLE_PIPELINE = "/api/DeploymentManager/v1/pipelines/{pipelineid}/disable"
API_ENDPOINT_ORCHESTRATOR_ENABLE_PIPELINE = "/api/DeploymentManager/v1/pipelines/{pipelineid}/enable"
API_ENDPOINT_ORCHESTRATOR_UPDATE_PIPELINE = "/api/DeploymentManager/v1/pipelines/{pipelineid}"
API_ENDPOINT_ORCHESTRATOR_DELETE_PIPELINE = "/api/DeploymentManager/v1/pipelines/{pipelineid}"

## Orchestrator - Deployment endpoints
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENTS = "/api/DeploymentManager/v1/pipelines/{pipelineid}/deployments"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENTS_LATEST = "/api/DeploymentManager/v1/pipelines/{pipelineid}/deployments?latest=true"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENTS_PENDING_PROMOTION = "/api/DeploymentManager/v1/pipelines/{pipelineid}/deployments?fetchDeploymentsInPendingPromotion=true"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENTS_OPEN_AND_PENDING_PROMOTION = "/api/DeploymentManager/v1/pipelines/{pipelineid}/deployments?fetchOpenAndPendingPromotionDeployments=true"
API_ENDPOINT_ORCHESTRATOR_TRIGGER_DEPLOYMENT = "/api/DeploymentManager/v1/pipelines/{pipelineid}/deployments"
API_ENDPOINT_ORCHESTRATOR_ABORT_DEPLOYMENT = "/api/DeploymentManager/v1/deployments/{id}/abort"
API_ENDPOINT_ORCHESTRATOR_PAUSE_DEPLOYMENT = "/api/DeploymentManager/v1/deployments/{id}/pause"
API_ENDPOINT_ORCHESTRATOR_RESUME_DEPLOYMENT = "/api/DeploymentManager/v1/deployments/{id}/resume"
API_ENDPOINT_ORCHESTRATOR_PROMOTE_DEPLOYMENT = "/api/DeploymentManager/v1/deployments/{id}/promote"
API_ENDPOINT_ORCHESTRATOR_RETRY_DEPLOYMENT = "/api/DeploymentManager/v1/deployments/{id}/retry"
API_ENDPOINT_ORCHESTRATOR_SKIP_DEPLOYMENT_TASK = "/api/DeploymentManager/v1/deployments/{id}/skip"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENT = "/api/DeploymentManager/v1/deployments/{id}"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENT_TASKS = "/api/DeploymentManager/v1/tasks"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENT_TASKS_PIPELINE = "/api/DeploymentManager/v1/tasks?PipelineID={pipeline_id}"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENT_TASKS_DEPLOYMENT = "/api/DeploymentManager/v1/tasks?DeploymentID={deployment_id}"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENT_TASKS_LATEST = "/api/DeploymentManager/v1/tasks?latest=true"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENT_TASKS_DEPLOYMENT_STATUS = "/api/DeploymentManager/v1/tasks?DeploymentID={deployment_id}&TaskStatus={task_status}"
API_ENDPOINT_ORCHESTRATOR_GET_DEPLOYMENT_TASK = "/api/DeploymentManager/v1/tasks/{id}"
API_ENDPOINT_ORCHESTRATOR_UPDATE_DEPLOYMENT_TASK = "/api/DeploymentManager/v1/tasks/{id}"
API_ENDPOINT_ORCHESTRATOR_QS_ENVIRONMENT_ID = '&EnvironmentID='



import PegaEnvironment
import json

class Candidate(PegaEnvironment.PegaEnvironment):
    import DeploymentManagerCandidateEndpoints as ENDPOINTS
    import DeploymentManagerCandidateOutputStrings as OUTPUTSTRINGS
    
    def __init__(self, system_name, environment_url, production_level):
        PegaEnvironment.PegaEnvironment.__production_level = production_level
        PegaEnvironment.PegaEnvironment.__init__(self, system_name, environment_url)

    def get_application(self, application_name: str, application_version: str):
        response = self.api_request_get(
            self.ENDPOINTS.API_ENDPOINT_CANDIDATE_GET_APPLICATION.format(
                appName=application_name, appVersion=application_version
            )
        )
        if response.ok:
            return json.loads(response.content)
        else:
            return None

    def get_ruleset(self, ruleset_name: str, ruleset_version: str):
        response = self.api_request_get(
            self.ENDPOINTS.api_request_getAPI_ENDPOINT_CANDIDATE_GET_RULESET.format(
                ruleset_name=ruleset_name, ruleset_version=ruleset_version
            )
        )
        if response.ok:
            return json.loads(response.content)
        else:
            return None

    def create_ruleset(self, ruleset_name: str, ruleset_version: str):
        response = self.api_request_post(
            self.ENDPOINTS.API_ENDPOINT_CANDIDATE_CREATE_RULESET.format(
                ruleset_name=ruleset_name, ruleset_version=ruleset_version
            ),
            "",
        )
        return response.ok

    def increase_ruleset_patch_version(self, ruleset_name: str):
        response = self.api_request_get(self.ENDPOINTS.API_ENDPOINT_CANDIDATE_CREATE_RULESET)
        return response.ok

    def increase_ruleset_minor_version(self, ruleset_name: str):
        response = self.api_request_get(self.ENDPOINTS.API_ENDPOINT_CANDIDATE_CREATE_RULESET)
        return response.ok
